@@ -181,8 +181,16 @@
 ;;
 
 (defsubst mucs-type-check-valid-serialization (serialize)
-  (if (not (or (listp serialize)
-	       (eq 'none serialize)))
+  (if (not (or (eq 'none serialize)
+	       t))
+;; Currently, this function is meaningless.
+;; The below checking may cause side-effects.
+;; I'd like to avoid it.
+;	       (condition-case nil
+;		   (progn
+;		     (eval serialize)
+;		     t)
+;		 (error nil))))
       (signal 'mucs-invalid-serialization-error serialize)))
 
 (defun mucs-type-register-serialization (type name serialize unserialize)
@@ -270,6 +278,12 @@
 
     (cdr conv-func-slot)))
 
+(defsubst mucs-type-get-conversion-force (from-type to-type)
+  "Get convertion from FROM-TYPE to TO-TYPE.
+If convertion cannot be found, cause error."
+  (or (mucs-type-get-conversion from-type to-type)
+      (signal 'mucs-type-cannot-convert (cons from-type to-type))))
+
 ;;;
 ;;; TYPE inspect functions
 ;;;
