@@ -1,5 +1,5 @@
 ;;; -*- coding: iso-2022-7bit; byte-compile-dynamic: t -*-
-;;; mucs.el --- for MULE-UCS configuration
+;;; mucs.el --- Mule-UCS setup file.
 
 ;; Copyright (C) 1997-2000 Miyashita Hisashi
 
@@ -26,9 +26,6 @@
 ;; Comment:
 
 ;;;; 
-;; 0.00->NUBATAMA (YUME)
-;; 0.50->MYOSHINJI Temple
-;; 0.60->ARASHIYAMA
 ;; 0.70->KITANOTENMANGU Shrine
 ;; 0.80->JISHOJI Temple
 ;; 0.90->Mt.DAIMONJI
@@ -43,18 +40,20 @@
 ;; 1.80->IMADEGAWA
 ;; 1.90->DAIRI
 ;; 2.00->Kyoto Univ.
-(defconst mucs-version "0.71 (KITANOTENMANGU:$BKLLnE7K~5\(B)")
+(defconst mucs-version "0.72 (RYOANJI:$BN50B;{(B)")
 
-;;
 ;; For error handling.
-;;
 
 (require 'mucs-error)
 
+;; Type manager.
+(require 'mucs-type)
+
 ;;; fundamental data.
 
 (defvar emacs-value-bits 28)
-(defvar mucs-code-range-bits 24)
+(defvar mucs-code-range-bits 27)
+(defvar mucs-code-range-specials 100)
 (defvar mucs-data-path "reldata/")
 (defvar mucs-invalid-code -1
   "invalid code.  If this value is set, skip operation.")
@@ -85,7 +84,17 @@
 	(require package))))
 
 (defun mucs-max-code ()
-  (1- (lsh 1 mucs-code-range-bits)))
+  (- (lsh 1 (- mucs-code-range-bits 2))
+     mucs-code-range-specials 1))
+
+(defun mucs-special-code (code)
+  (if (or (< code 0)
+	  (>= code mucs-code-range-specials))
+      (error "Invalid code:%d" code))
+  (+ (mucs-max-code) code 1))
+
+(defun mucs-code-range-mask ()
+  (- (lsh 1 (1- mucs-code-range-bits)) 1))
 
 (defun mucs-number-mask ()
   (lognot 0))
@@ -118,6 +127,37 @@ This variable specifies the type of data that the current context stores.")
   "At the end of package definition, call this hook.
 In order to embed data or lisp code, use this hook.")
 
+(defmacro mucs-embed-package-signature ()
+  (let ((packages 
+	 (cons mucs-current-package
+	       (get mucs-current-package 'mucs-imported-packages)))
+	(sig '(progn))
+	cont result tempfunc)
+    (setq tempfunc
+	  (lambda (package)
+	    (setq cont (get package
+			    'mucs-registered-alist))
+	    (setq result
+		  (if cont
+		      `((put (quote ,package)
+			     'mucs-registered-alist
+			     (quote ,cont)))
+		    nil))
+	    (setq cont (get package
+			    'mucs-imported-packages))
+	    (if cont
+		(setq result
+		      (append
+		       `((put (quote ,package)
+			      'mucs-imported-packages
+			      (quote ,cont)))
+		       result)))
+	    result))
+    (while packages
+      (setq sig (append sig (funcall tempfunc (car packages)))
+	    packages (cdr packages)))
+    sig))
+
 (defmacro mucs-embed-program-with-hooks (hooksym)
   (let ((hookval (symbol-value hooksym))
 	result)
@@ -147,49 +187,109 @@ that you set at `provide' function."
    `(let ((mucs-current-package (quote ,package))))
    form
    '((mucs-embed-program-with-hooks
-      mucs-package-definition-end-hook))))
+      mucs-package-definition-end-hook)
+     (mucs-embed-package-signature))))
 
-(defsubst mucs-registered-p (kind object)
-  "If OBJECT have been already registered, return non-nil."
-  (memq object 
-	(cdr
-	 (assq kind
-	       (get mucs-current-package
-		    'mucs-registered-alist)))))
+(defmacro mucs-import-package (package)
+  "Import package."
+  (require package)
+  (let ((import-list
+	 (get mucs-current-package 'mucs-imported-packages)))
+    (if (memq package import-list)
+	nil
+      (put mucs-current-package 'mucs-imported-packages
+	   (cons package import-list)))
+    `(require (quote ,package))))
 
-(defun mucs-register-object (kind object)
+(defsubst mucs-get-current-registered-alist ()
+  (get mucs-current-package
+       'mucs-registered-alist))
+
+(defsubst mucs-set-current-registered-alist (alist)
+  (put mucs-current-package
+       'mucs-registered-alist
+       alist))
+
+(defsubst mucs-get-registered-kind-alist (kind)
+  (let ((packages 
+	 (cons mucs-current-package
+	       (get mucs-current-package 'mucs-imported-packages)))
+	result)
+    (while packages
+      (setq result
+	    (append
+	     (cdr (assq kind
+			(get (car packages)
+			     'mucs-registered-alist)))
+	     result)
+	    packages (cdr packages)))
+    result))
+
+(defsubst mucs-get-registered-slot (kind object)
+  "If OBJECT have been already registered, return registered slot."
+  (assq object
+	(mucs-get-registered-kind-alist kind)))
+
+(defalias 'mucs-registered-p 'mucs-get-registered-slot)
+
+(defsubst mucs-embedded-p (kind object)
+  (nth 1 (mucs-get-registered-slot kind object)))
+
+(defun mucs-registered-object-list (kind)
+  (let ((objlist
+	 (mucs-get-registered-kind-alist kind))
+	elem result)
+    (while (setq elem (car objlist))
+      (setq result (cons (car elem) result)
+	    objlist (cdr objlist)))
+    result))
+
+(defun mucs-unembedded-object-list (kind)
+  (let ((objlist
+	 (mucs-get-registered-kind-alist kind))
+	elem result)
+    (while (setq elem (car objlist))
+      (if (not (cdr elem))
+	  (setq result (cons (car elem) result)))
+      (setq objlist (cdr objlist)))
+    result))
+
+(defun mucs-notify-embedment (kind object)
+  (let ((slot (mucs-get-registered-slot kind object)))
+    (if (null slot)
+	(error "%S has not been registered yet.(KIND:%S)"
+	       object kind))
+    (setcar (cdr slot) t)))
+
+(defun mucs-register-object (kind object &optional embed)
   "Register OBJECT to curent package's record.
 If OBJECT have been already registered, return non-nil;
 otherwise return nil."
-  (if (and mucs-current-package
-	   (symbolp mucs-current-package))
-      (or
-       (mucs-registered-p kind object)
-       (let* ((alist
-	       (get mucs-current-package
-		    'mucs-registered-alist))
-	      (slot
-	       (assq kind alist)))
-	 (if slot
-	     (setcdr slot
-		     (cons object (cdr slot)))
-	   (put mucs-current-package
-		'mucs-registered-alist
-		(cons (list kind object)
-		      alist)))
-	 nil))))
+  (if mucs-current-package
+      (or (mucs-registered-p kind object)
+	  (let* ((alist
+		  (mucs-get-current-registered-alist))
+		 (slot
+		  (assq kind alist))
+		 (objslot (list object embed)))
+	    (if slot
+		(setcdr slot
+			(cons objslot (cdr slot)))
+	      (mucs-set-current-registered-alist
+	       (cons (list kind objslot)
+		     alist)))
+	    nil))))
 
 (defun mucs-unregister-object (kind object)
-  (if (and mucs-current-package
-	   (symbolp mucs-current-package))
-      (let* ((alist
-	      (get mucs-current-package
-		   'mucs-registered-alist))
-	     (slot
-	      (assq kind alist)))
-	(if slot
-	    (setcdr slot
-		    (delq object (cdr slot)))))))
+  (let* ((alist
+	  (mucs-get-current-registered-alist))
+	 (slot1
+	  (assq kind alist))
+	 slot2)
+    (and slot1
+	 (setq slot2 (assq object slot1))
+	 (setcdr slot1
+		 (delq slot2 (cdr slot1))))))
 
 ;;; Fundamental configuration ends here.
 
