@@ -19,7 +19,8 @@
 
 ;;; Code:
 
-(put 'chinese-cns11643-4 'unicode 'cns11643-4-vs-unicode-assoc)
+(put 'chinese-cns11643-4 'unicode-assoc
+     'cns11643-4-vs-unicode-assoc)
 
 (defvar cns11643-4-vs-unicode-assoc
   '(assoc
