@@ -1,9 +1,9 @@
 ;;; -*- coding: iso-2022-7bit  -*-
-;;; tae.el --- Translation rule And Encoding compiler(TAE:$BL/(B):-)
+;;; tae.el --- Translation And Encoding compiler(TAE:$BL/(B):-)
 
 ;; Copyright (C) 1997-2000 Miyashita Hisashi
 
-;; Keywords: mule, multilingual, encode, CCL
+;; Keywords: mule, multilingual, encoder
 
 ;; This file is part of Mule-UCS
 
@@ -30,13 +30,60 @@
 (require 'mucs-type)
 (require 'tbl-mg)
 
+;;; TAE control variables
+
+(defvar tae-enable-register-translation t
+  "This variable controls whether TAE register translation to package.")
+
+
 ;;; primitive data for translation.
 
-(defun tae-project-all-translation-vector (val)
-  (vector t
-	  (if (eq val 'invalid)
-	      mucs-invalid-code val)
-	  0 (mucs-max-code)))
+(defun tae-generate-project-table-symbol (val)
+  (cond ((numberp val)
+	 (intern (format "tae-project-all-to-%d" val)))
+	((eq val 'invalid)
+	 'tae-project-all-to-invalid)
+	(t
+	 (signal 'wrong-type-argument val))))
+
+(defun tae-generate-special-project-table-symbol (code)
+  (if (numberp code)
+      (intern (format "tae-project-special-%d-to-identity" code))
+    (signal 'wrong-type-argument code)))
+
+(defun tae-project-all-table (val)
+  (let ((sym (tae-generate-project-table-symbol val)))
+    (if (get sym 'code-conversion-map)
+	sym
+      (register-code-conversion-map
+       sym
+       (vector t
+	       (if (eq val 'invalid)
+		   mucs-invalid-code val)
+	       0 (mucs-max-code)))
+      sym)))
+
+(defun tae-project-special-to-identity-table (code)
+  (let ((sym (tae-generate-project-table-symbol code)))
+    (if (get sym 'code-conversion-map)
+	sym
+      (register-code-conversion-map
+       sym
+       (vector (mucs-special-code code) t)))))
+
+(defun tae-project-all-specials-to-identity-table ()
+  (if (get 'tae-project-all-special-to-identity
+	   'code-conversion-map)
+      'tae-project-all-special-to-identity
+    (register-code-conversion-map
+     'tae-project-all-special-to-identity
+     (vector t
+	     (mucs-special-code 0)
+	     (mucs-special-code
+	      (1- mucs-code-range-specials))))))
+
+(defun tae-project-all-to-special-table (code)
+  (tae-project-all-table (mucs-special-code code)))
 
 ;;; generate unique symbol
 
@@ -101,10 +148,11 @@ which is managed by TAE."
 ;;                   Produced TAE messages for encode.
 ;;               tae-products-for-decode:
 ;;                   Produced TAE messages for decode.
-;;               tae-table-set:
+;;               tae-table-set-for-encode:
+;;               tae-table-set-for-decode:
 ;;                   Generated table-sets.
-;;                   This is an A-list whose key is
-;;                   (TYPE . (ENCODE-TABLE-SET . DECODE-TABLE-SET))
+;;                   Each has an A-list whose key is
+;;                   (TYPE . TABLE-SET)
 
 (defvar tae-internal-translation-property
   '(tae-translation
@@ -113,7 +161,9 @@ which is managed by TAE."
     tae-reduced-translation-for-decode
     tae-products-for-encode
     tae-products-for-decode
-    tae-table-set
+;;
+    tae-table-set-for-encode
+    tae-table-set-for-decode
 ;;
     tae-dependent-translations
     tae-dependent-conversions))
@@ -131,49 +181,51 @@ which is managed by TAE."
      (put dest x (get src x)))
    tae-internal-translation-property)
   ;;; declared table-set must be copied.
-  (let ((table-sets (get src 'tae-table-set))
+  (let ((table-sets (append (get src 'tae-table-set-for-encode)
+			    (get src 'tae-table-set-for-decode)))
 	new-table-set-name)
     (while table-sets
       (table-set-add-reference (car table-sets))
       (setq table-sets (cdr table-sets)))))
 
 (defun tae-translation-add-table-set (name table-set type decodep)
-  (let* ((alist (get name 'tae-table-set))
-	 (maybe (cdr (assq type alist))))
-    (if (and maybe
-	     (if decodep
-		 (cdr maybe)
-	       (car maybe)))
+  (let* ((sym (if decodep
+		  'tae-table-set-for-decode
+		'tae-table-set-for-encode))
+	 (alist (get name sym))
+	 (maybe (assq type alist)))
+    (if (cdr maybe)
 	(error "Translation %S has already have table-set:%S(TYPE:%S, DECODEP:%S)."
 	       name maybe type decodep))
     (if maybe
-	(if decodep
-	    (setcdr maybe table-set)
-	  (setcar maybe table-set))
-      (put name 'tae-table-set
-	   (cons (cons type
-		       (if decodep
-			   (cons nil table-set)
-			 (cons table-set nil)))
+	(setcdr maybe table-set)
+      (put name sym
+	   (cons (cons type table-set)
 		 alist)))))
 
 (defsubst tae-translation-get-table-set (name type decodep)
   (if decodep
-      (cdr (cdr (assq type (get name 'tae-table-set))))
-    (car (cdr (assq type (get name 'tae-table-set))))))
+      (cdr (assq type (get name 'tae-table-set-for-decode)))
+    (cdr (assq type (get name 'tae-table-set-for-encode)))))
 
 (defun tae-initialize-translation (name)
   (set name nil)
 
+  ;;; Initialize internal properties.
+  (mapcar
+   (lambda (x)
+     (put name x nil))
+   tae-internal-translation-property)
+
   ;;; Remove references to all generated table-sets.
   (mapcar
    (lambda (x)
-     (setq x (cdr x))
-     (if (car x)
-	 (table-set-remove-reference (car x)))
-     (if (cdr x)
-	 (table-set-remove-reference (cdr x))))
-   (get name 'tae-table-set)))
+     (table-set-remove-reference (cdr x)))
+   (get name 'tae-table-set-for-encode))
+  (mapcar
+   (lambda (x)
+     (table-set-remove-reference (cdr x)))
+   (get name 'tae-table-set-for-decode)))
 
 (defsubst tae-get-translation-produced-products (translation type decodep)
   (cdr (assq type
@@ -196,19 +248,47 @@ which is managed by TAE."
 
 ;;; registered translation manager
 
-(defun tae-register-translation (name)
-  (let ((trlist (get mucs-current-package
-		     'tae-registered-translation)))
-    (if (not (memq name trlist))
-	(progn
-	  (setq trlist (cons name trlist))
-	  (put mucs-current-package
-	       'tae-registered-translation
-	       trlist)))))
+(defun tae-register-translation (name type decodep)
+  (if tae-enable-register-translation
+      (progn
+	(mucs-register-object 'tae-translation name)
+	(let* ((n (if decodep 3 2))
+	       (slot (mucs-get-registered-slot
+		      'tae-translation name))
+	       (typelist (nth n slot))
+	       (typeslot (assq type typelist)))
+	  (if typeslot
+	      nil
+	    (if (< (length slot) 4)
+		(setcdr (nthcdr 1 slot) (list nil nil)))
+	    (setcar (nthcdr n slot) (cons (list type nil) typelist)))))))
 
-(defun tae-retrieve-registered-translation-list ()
-  (get mucs-current-package
-       'tae-registered-translation))
+(defsubst tae-registered-translation-rule-list ()
+  (mucs-registered-object-list 'tae-translation))
+
+(defun tae-notify-embedment-product-list (name product-list decodep)
+  (let* ((slot (mucs-get-registered-slot 'tae-translation name))
+	 (prlist (nth (if decodep 3 2) slot))
+	 elem rec)
+    (while (setq elem (car product-list))
+      (setq rec (assq elem prlist)
+	    product-list (cdr product-list))
+      (if (null rec)
+	  (error "TAE Internal error:%S is not product of %S"
+		 elem name))
+      (setcar (cdr rec) t)))
+  nil)
+
+(defun tae-embedment-required-product-list (name decodep)
+  (let* ((slot (mucs-get-registered-slot 'tae-translation name))
+	 (prlist (nth (if decodep 3 2) slot))
+	 elem result)
+;    (message "NAME:%S PL:%S" name prlist)
+    (while (setq elem (car prlist))
+      (if (not (nth 1 elem))
+	  (setq result (cons (car elem) result)))
+      (setq prlist (cdr prlist)))
+    result))
 
 (defun tae-register-dependent-translation (dep-name)
   (let ((trlist (get tae-current-compiling-translation
@@ -240,9 +320,11 @@ which is managed by TAE."
 
 ;;; translation declaration.
 
-(defun tae-declare-translation (name translation &optional dynamicp)
+(defun tae-declare-translation (name translation &optional holdp dynamicp)
   "Declare translation.
 NAME must be a symbol to distinguish TRANSLATION.
+If HOLDP is non-nil, the declared translation is not initialized on
+already produced products and others.
 If DYNAMICP is non-nil, this translation can be modified
 after its compilation."
   (if (not (symbolp name))
@@ -250,7 +332,8 @@ after its compilation."
 
   (if (tae-declared-translation-p translation)
       (tae-copy-declared-translation name translation)
-    (tae-initialize-translation name)
+    (if (not holdp)
+	(tae-initialize-translation name))
     (set name translation)
     (put name 'tae-translation t)
     (if dynamicp
@@ -986,14 +1069,32 @@ according to it, rearrange the elements of UNIT."
 (defsubst tae-message-put (message key value)
   (setcdr (assq key message) value))
 
-(defsubst tae-message-add-tables (message tables)
-  (let ((slot (assq 'tables message)))
-    (if slot
+(defsubst tae-get-message-from-translation (tr type decodep)
+  (let ((result (copy-sequence
+		 (tae-get-compiled-products-internal tr type decodep)))
+	tables-new-slot)
+    ;;; defined table-set.
+    (if (tae-declared-translation-p tr)
 	(progn
-	  (setcdr slot
-		  (nconc (cdr slot) tables))
-	  message)
-      (cons (cons 'tables tables) message))))
+	  (if (assq 'tables result)
+	      (error "Internal Error:message(%S)" result))
+	  (cons result
+		(list
+		 (tae-translation-get-table-set tr type decodep))))
+      ;;; generated 'tables
+      (if (setq tables-new-slot (assq 'tables result))
+	  (cons
+	   (delq tables-new-slot result)
+	   (cdr tables-new-slot))))))
+
+(defsubst tae-message-set-tables (message tables)
+  (let ((tables-slot (assq 'tables message)))
+    (if tables-slot
+	(setcdr tables-slot tables)
+      (setq message (cons
+		     (cons 'tables tables)
+		     message)))
+    message))
 
 (defsubst tae-message-set-new-type (message type)
   (let ((slot (assq 'new-type message)))
@@ -1004,6 +1105,7 @@ according to it, rearrange the elements of UNIT."
       (cons (cons 'new-type type) message))))
 
 (defun tae-generate-new-table-set (tables name type decodep)
+;  (message "NAME:%S TYPE:%S DECODEP:%S" name type decodep)
   (let (table-set)
     (while
 	(progn
@@ -1064,7 +1166,7 @@ according to it, rearrange the elements of UNIT."
 			  conv-slot)
 			 (if tr-elem-all-key
 			     (list
-			      (tae-project-all-translation-vector
+			      (tae-project-all-table
 			       tr-elem-all-key))
 			   nil)))))
 
@@ -1078,46 +1180,94 @@ according to it, rearrange the elements of UNIT."
     (tae-message-set-new-type result to-type)))
 
 (defun tae-compile-OR-operation (translations type decodep)
-  (let (tr result results to-type new-type
-	   table-set-new-slot tables-new-slot)
+  (let (tr mes-tbl mes tbls result-mes to-type new-type)
     (while (setq tr (car translations))
       (setq translations (cdr translations)
-	    result (copy-sequence
-		    (tae-get-compiled-products-internal tr type decodep))
-	    new-type (cdr (assq 'new-type result)))
+	    mes-tbl (tae-get-message-from-translation
+		     tr type decodep)
+	    mes (car mes-tbl)
+	    tbls (nconc tbls (cdr mes-tbl))
+	    new-type (cdr (assq 'new-type mes)))
 
       ;;; operate type.
       (if to-type
 	  (if (not (eq to-type new-type))
 	      (signal 'mucs-type-mismatch-error to-type new-type))
 	(setq to-type new-type)
-	(setq results
-	      (tae-message-set-new-type results to-type)))
-      ;;; operate table-set.
-      (if (and (tae-declared-translation-p tr)
-	       (setq table-set-new-slot
-		     (tae-translation-get-table-set tr type decodep)))
-	  (progn
-	    (setq results (tae-message-add-tables
-			   results (list table-set-new-slot)))))
-      ;;; operate 'tables key.
-      (if (setq tables-new-slot (assq 'tables result))
-	  (progn
-	    (setq results (tae-message-add-tables
-			   results (cdr tables-new-slot)))
-	    (setq result (delq tables-new-slot result))))
+	(setq result-mes
+	      (tae-message-set-new-type result-mes to-type)))
 
-      ;; any other message keys are preserved.
-      (setq results (tae-message-append results result)))
-    results))
+      ;; normal message keys are preserved.
+      (setq result-mes (tae-message-append result-mes mes)))
+    (tae-message-set-tables result-mes
+			    tbls)))
 
 (defun tae-compile-AND-operation (translations type decodep)
-  (error "Not supported yet:"))
+  (let (tr mes-tbl mes tbls result-mes to-type new-type)
+    (while (setq tr (car translations))
+      (setq translations (cdr translations)
+	    mes-tbl (tae-get-message-from-translation
+		     tr type decodep)
+	    mes (car mes-tbl)
+	    new-type (cdr (assq 'new-type mes)))
+
+      ;;; operate type.
+      (if to-type
+	  (if (not (eq to-type new-type))
+	      (signal 'mucs-type-mismatch-error to-type new-type))
+	(setq to-type new-type)
+	(setq result-mes
+	      (tae-message-set-new-type result-mes to-type)))
+
+      (setq tbls (nconc (list
+			 (list (nconc (cdr mes-tbl)
+				      (list (tae-project-all-to-special-table 1)))
+			       (tae-project-all-table t))
+			 (tae-project-special-to-identity-table 1))
+			tbls))
+
+      ;; normal message keys are preserved.
+      (setq result-mes (tae-message-append result-mes mes)))
+    (tae-message-set-tables result-mes
+			    tbls)))
 
 (defun tae-compile-composite-operation (translations type decodep)
-  (error "Not supported yet:"))
+  (let (tr mes-tbl mes tbls result-mes)
+    (while (setq tr (car translations))
+      (setq translations (cdr translations)
+	    mes-tbl (tae-get-message-from-translation
+		     tr type decodep)
+	    tbls (cons (cdr mes-tbl) tbls)
+	    mes (car mes-tbl)
+	    type (cdr (assq 'new-type mes)))
+
+      ;; normal message keys are preserved.
+      (setq result-mes (tae-message-append result-mes mes)))
+    (tae-message-set-tables
+     (tae-message-set-new-type result-mes type)
+     (nreverse tbls))))
+
 (defun tae-compile-composite-transparent-operation (translations type decodep)
-  (error "Not supported yet:"))
+  (if (/= (length translations) 2)
+      (error "Composite Transparent requires two arguments:%S"
+	     translations))
+  (let ((mes-tbl-a (tae-get-message-from-translation
+		    (car translations) type decodep))
+	mes-tbl-b mes tbls result-mes)
+    (setq mes (car mes-tbl-a)
+	  type (cdr (assq 'new-type mes))
+	  mes-tbl-b (tae-get-message-from-translation
+		     (nth 1 translations) type decodep)
+	  tbls (cons (cdr mes-tbl-a)
+		     (nconc (cdr mes-tbl-b)
+			    (list (tae-project-all-table t))
+			    (cdr mes-tbl-a))))
+
+    ;; normal message keys are preserved.
+    (setq result-mes (tae-message-append mes (car mes-tbl-b)))
+    (tae-message-set-tables
+     (tae-message-set-new-type result-mes type)
+     tbls)))
 
 (defun tae-get-compiled-products-internal (translation type decodep)
   (let (op args)
@@ -1158,8 +1308,8 @@ according to it, rearrange the elements of UNIT."
   (let ((products (tae-get-translation-produced-products
 		   name type decodep))
 	table-set tables-slot translation)
-    ;; In order to make trasnaltion data persistent.
-    (tae-register-translation name)
+    ;; In order to make traslation data persistent.
+    (tae-register-translation name type decodep)
 
     (if products
 	products
@@ -1196,67 +1346,121 @@ according to it, rearrange the elements of UNIT."
 ;;; to manipulate the tranlation rule is kept in
 ;;; byte-compiled file.
 
-(defun tae-generate-persistent-program (name persistent-items
-					     &optional subtype)
+(defun tae-persistent-program-template (name item objs replacep)
+  ;;(message "NAME:%S ITEM:%S OBJS:%S" name item objs)
+  (if replacep
+      `(put (quote ,name)
+	    (quote ,item)
+	    (quote ,objs))
+    `(put (quote ,name)
+	  (quote ,item)
+	  (nconc (quote ,objs)
+		 (get (quote ,name)
+		      (quote ,item))))))
+
+(defun tae-generate-persistent-program (name persistent-items)
   "Generate programs to make translation NAME persistent."
   (if (not (tae-declared-translation-p name))
       (error "NAME:%S is not declared translation rule." name))
-  (let (result current-item elem object)
+  (let (result current-item objs objlist replacep
+	       product-list clist)
     (while (setq current-item (car persistent-items))
       (setq persistent-items (cdr persistent-items))
       (if (eq current-item 'tae-translation-definition)
-	  (and 
-	     ;;; See below note!!!
-;;	     (not (mucs-register-object current-item elem))
-	     (not (mucs-register-object name current-item))
-	     (setq result
-		   (cons
-		    `(setq ,name (quote ,(symbol-value name)))
-		    result)))
-	(setq elem
-	      (cond ((memq current-item
-			   '(tae-translation
-			     tae-dynamic-translation
-			     tae-dependent-conversions))
-		     (get name current-item))
+	  (setq result
+		(cons
+		 `(setq ,name (quote ,(symbol-value name)))
+		 result))
+	(cond ((memq current-item
+		     '(tae-translation
+		       tae-dynamic-translation
+		       tae-dependent-conversions))
+	       (setq objs (get name current-item)
+		     replacep t))
 
-		    ((eq current-item
-			 'tae-reduced-translation-for-encode)
-		     (tae-reduce-translation name nil))
-		    ((eq current-item
-			 'tae-reduced-translation-for-decode)
-		     (tae-reduce-translation name t))
+	      ((eq current-item
+		   'tae-reduced-translation-for-encode)
+	       (setq objs (tae-reduce-translation name nil)
+		     replacep t))
 
-		    ((memq current-item
-			   '(tae-products-for-encode
-			     tae-products-for-decode
-			     tae-table-set))
-		     (if subtype
-			 (tae-get-compiled-products
-			  name
-			  (if (eq current-item 
-				  'tae-products-for-decode)
-			      t nil)
-			  subtype)
-		       (get name current-item)))
+	      ((eq current-item
+		   'tae-reduced-translation-for-decode)
+	       (setq objs (tae-reduce-translation name t)
+		     replacep t))
+
+	      ((eq current-item
+		   'tae-products-for-encode)
+	       (setq replacep nil
+		     product-list (tae-embedment-required-product-list
+				   name nil)
+		     clist product-list
+		     objs nil
+		     objlist (get name 'tae-products-for-encode))
+	       (while clist
+		 (setq objs (cons (assq (car clist) objlist)
+				  objs)
+		       clist (cdr clist)))
+	       (if objs 
+		   (setq result
+			 (cons (tae-persistent-program-template
+				name 'tae-products-for-encode objs nil)
+			       result)))
+	       (setq objs nil
+		     objlist (get name 'tae-table-set-for-encode)
+		     clist product-list)
+	       (while clist
+		 (setq objs (cons (assq (car clist) objlist)
+				  objs)
+		       clist (cdr clist)))
+	       (if objs
+		   (setq result
+			 (cons (tae-persistent-program-template
+				name 'tae-table-set-for-encode objs nil)
+			       result)))
+	       (tae-notify-embedment-product-list
+		name product-list nil))
+
+	      ((eq current-item
+		   'tae-products-for-decode)
+	       (setq replacep nil
+		     product-list (tae-embedment-required-product-list
+				   name t)
+		     clist product-list
+		     objs nil
+		     objlist (get name 'tae-products-for-decode))
+	       (while clist
+		 (setq objs (cons (assq (car clist) objlist)
+				  objs)
+		       clist (cdr clist)))
+	       (if objs
+		   (setq result
+			 (cons (tae-persistent-program-template
+				name 'tae-products-for-decode objs nil)
+			       result)))
+	       (setq objs nil
+		     objlist (get name 'tae-table-set-for-decode)
+		     clist product-list)
+	       (while clist
+		 (setq objs (cons (assq (car clist) objlist)
+				  objs)
+		       clist (cdr clist)))
+	       (if objs
+		   (setq result
+			 (cons (tae-persistent-program-template
+				name 'tae-table-set-for-decode objs nil)
+			       result)))
+	       (tae-notify-embedment-product-list name product-list t))
+
+	      (t
+	       (error "%S is not valid item that can be persistent."
+		      current-item)))
+	(if (and objs
+		 replacep)
+	    (setq result
+		  (cons (tae-persistent-program-template
+			 name current-item objs t)
+			result)))))
 
-		    (t
-		     (error "%S is not valid item that can be persistent."
-			    current-item))))
-	(and elem
-	     ;;; Registering objects derived from translation-rule.
-	     ;;; The first line register objects generated by
-	     ;;; translation-rule as is.  On the contrary, the
-	     ;;; second line register symbol that identify generated
-	     ;;; objects.  It depends on object identification which one
-	     ;;; should be chosen.
-;;	     (not (mucs-register-object current-item elem))
-	     (not (mucs-register-object name current-item))
-	     (setq result
-		   (cons `(put (quote ,name)
-			       (quote ,current-item)
-			       (quote ,elem))
-			 result)))))
 ;    (message "TR:%S,Result:%S" name result)
     result
 ;;; enclose with `progn' form?
@@ -1307,11 +1511,17 @@ according to it, rearrange the elements of UNIT."
 (defun tae-compile-retrieve-ccl-prog (table-syms nestedp)
   (if nestedp
       `(loop
+	(r4 = r0)
 	(r1 = 0)
-	(map-multiple r1 r0 ,table-syms))
+	(map-multiple r1 r0 ,table-syms)
+	(if (r0 > ,(mucs-max-code))
+	    (r0 = ((r4 + r0) & ,(mucs-code-range-mask)))))
     `(loop
+      (r4 = r0)
       (r1 = 0)
-      (iterate-multiple-map r1 r0 ,@table-syms))))
+      (iterate-multiple-map r1 r0 ,@table-syms)
+      (if (r0 > ,(mucs-max-code))
+	  (r0 = ((r4 + r0) & ,(mucs-code-range-mask)))))))
 
 ;; API for compilation.
 
@@ -1344,8 +1554,8 @@ NAME must be a declared translation."
   (if (not (tae-translation-dynamic-p name))
       (error "%S cannot be modified dynamically!"
 	     name))
-  (let ((conversion-list (get name 'tae-dependent-conversions))
-	(result-myo (mucs-ccl-empty-myo))
+  (let ((tae-enable-register-translation nil)
+	(conversion-list (get name 'tae-dependent-conversions))
 	intdata	slot conv root-tr mark type decodep)
     (set name rule)
     (mapcar
@@ -1380,6 +1590,54 @@ NAME must be a declared translation."
 	(nth 1 intdata)
 	(nth 2 intdata))))))
 
+;;; slightly ad-hoc.
+;;; We should separate mucs-ccl and setup part to other modules.
+(defmacro tae-embed-for-dynamic-modification (name rule)
+  (setq name (eval name)
+	rule (eval rule))
+  (if (not (tae-translation-dynamic-p name))
+      (error "%S cannot be modified dynamically!"
+	     name))
+  (let ((conversion-list (get name 'tae-dependent-conversions))
+	cdata-myo rdata-myo slot conv root-tr mark type decodep
+	result)
+    (set name rule)
+    (mapcar
+     (lambda (x)
+       (put name x nil))
+     '(tae-reduced-translation-for-encode
+       tae-reduced-translation-for-decode
+       tae-products-for-encode
+       tae-products-for-decode))
+    (while conversion-list
+      (setq slot (car conversion-list)
+	    conv (car slot)
+	    root-tr (nth 1 slot)
+	    mark (nth 2 slot)
+	    type (nth 3 slot)
+	    decodep (nth 4 slot)
+	    conversion-list (cdr conversion-list))
+
+      ;; reconstruct translation products.
+      (setq cdata-myo
+	    (car (tae-compile-internal name decodep type)))
+      ; flush generated products.
+;       (put root-tr
+; 	   (if decodep
+; 	       'tae-products-for-decode
+; 	     'tae-products-for-encode)
+; 	   nil)
+      (setq rdata-myo
+	    (car (tae-compile-internal root-tr decodep type)))
+      (setq result
+	    (append result
+		    (mucs-ccl-make-elisp-preparation-from-myo
+		     cdata-myo)
+		    (mucs-ccl-make-elisp-preparation-from-myo
+		     rdata-myo))))
+    (cons 'progn
+	  result)))
+
 ;; MAPPING FUNCTION
 ;;
 ;;  mapping functions are 
@@ -1463,25 +1721,28 @@ a list<CDR> (TABLE<SYMBOL> TABLE<SYMBOL> ...)."
 ;;; generate registration program
 
 (defun tae-generate-registration-program ()
-  (let ((translation-list (tae-retrieve-registered-translation-list))
+  (let ((translation-list (tae-registered-translation-rule-list))
 	tr result)
     (while (setq tr (car translation-list))
+;      (message "TR:%S EB:%S" tr (mucs-embedded-p 'tae-translation tr))
       (setq result
 	    (append
 	     (tae-generate-persistent-program
 	      tr
-	      (if (tae-translation-dynamic-p tr)
-		  '(tae-translation-definition
-		    tae-translation
-		    tae-dynamic-translation
+	      (if (mucs-embedded-p 'tae-translation tr)
+		  '(tae-products-for-encode
+		    tae-products-for-decode)
+		(mucs-notify-embedment 'tae-translation tr)
+		(if (tae-translation-dynamic-p tr)
+		    '(tae-translation-definition
+		      tae-translation
+		      tae-dynamic-translation
+		      tae-products-for-encode
+		      tae-products-for-decode
+		      tae-dependent-conversions)
+		  '(tae-translation
 		    tae-products-for-encode
-		    tae-products-for-decode
-		    tae-dependent-conversions
-		    tae-table-set)
-		'(tae-translation
-		  tae-products-for-encode
-		  tae-products-for-decode
-		  tae-table-set)))
+		    tae-products-for-decode))))
 	     result)
 	    translation-list
 	    (cdr translation-list)))
