@@ -67,6 +67,9 @@ for specifying a temporary table set.")
        (or (get table-set 'table-set-definition)
 	   (get table-set 'table-set-symbol-list))))
 
+(defsubst table-set-table-p (table)
+  (get table 'code-conversion-map))
+
 (defun table-set-add-reference (table-set)
   "Add reference count of table-set.
 If you want to persist the table-set, call this."
@@ -91,24 +94,28 @@ If you don't need the table-set, call this."
     (setq def (cdr def))))
 
 (defun make-table-set-definition-from-symbol-list (table-set)
-  (let (make-func current)
-    (setq make-func
-	  (lambda (syms result)
-	    (while (setq current (car syms))
-	      (setq result (cons
-			    (cond ((listp current)
-				   (funcall make-func current nil))
-				  ((table-set-p current)
-				   current)
-				  (t
-				   (get current 'code-conversion-map)))
-			    result)
-		    syms (cdr syms)))
-	    (nreverse result)))
-    (put table-set
-	 'table-set-definition
-	 (funcall make-func (get table-set 'table-set-symbol-list)
-		  nil))))
+  (error "Internal Error.
+It is disabled to make definition from symbol list.")
+;   (let (make-func)
+;     (setq make-func
+; 	  (lambda (syms)
+; 	    (let (result current)
+; 	      (while (setq current (car syms))
+; 		(setq result (cons
+; 			      (cond ((listp current)
+; 				     (funcall make-func current))
+; 				    ((table-set-p current)
+; 				     current)
+; 				    (t
+; 				     (get current 'code-conversion-map)))
+; 			      result)
+; 		      syms (cdr syms)))
+; 	      (nreverse result))))
+;     (put table-set
+; 	 'table-set-definition
+; 	 (funcall make-func (get table-set 'table-set-symbol-list)
+; 		  )))
+)
 
 (defsubst get-table-set-definition (table-set)
   (or (get table-set 'table-set-definition)
@@ -123,31 +130,22 @@ Definition must be a (nested) list of tables."
   (put table-set 'table-set-definition definition)
   (table-set-definition-add-reference-to-table-set definition))
 
-(defun table-set-add-tables (table-set tables)
-  ;;; this operation make table-set-symbol-list obsolete.
-  (put table-set 'table-set-symbol-list nil)
-  (table-set-definition-add-reference-to-table-set tables)
-  (put table-set 'table-set-definition
-       (append (get-table-set-definition table-set)
-	       tables)))
-
 (defun table-set-nested-p (table-set)
   "Inspect whether this table-set is nested or not.
 Nested table-set have at least one list that is consists of tables
 or lists of tables."
   (let ((flag (get table-set 'table-set-nested))
-	cur-def
-	definition)
+	cur symlist)
     (if flag
 	(eq flag 'nested)
-      (setq definition (get-table-set-definition table-set))
-      (while (and (setq cur-def (car definition))
-		  (not (consp cur-def))
-		  (or (vectorp cur-def)
-		      (and (table-set-p cur-def)
-			   (not (table-set-nested-p cur-def)))))
-	(setq definition (cdr definition)))
-      (if definition
+      (setq symlist (get-table-set-symbol-list table-set))
+      (while (and (setq cur (car symlist))
+		  (not (consp cur))
+		  (symbolp cur)
+		  (not (and (table-set-p cur)
+			    (table-set-nested-p cur))))
+	(setq symlist (cdr symlist)))
+      (if symlist
 	  (progn
 	    (put table-set 'table-set-nested 'nested)
 	    t)
@@ -185,32 +183,33 @@ used by the specified table-set."
 	func-get-syms
 	elem tables table-sym sym-el)
     (setq func-get-syms
-	  (lambda (x result)
-	    (while (setq elem (car x))
-	      (cond  ((vectorp elem)
-		      (setq table-sym
-			    (generate-table-set-table-symbol table-set i))
-		      (register-code-conversion-map table-sym (car x))
-		      (setq result (cons table-sym result)
-			    i (1+ i)))
-		     ((listp elem)
-		      (setq result (cons
-				    (funcall func-get-syms elem nil)
-				    result)))
-		     ((table-set-p elem)
-		      (setq result (cons elem result)))
-		     (t
-		      (error
-		       "Internal Error:invalid table-set definition:%S"
-		       elem)))
-	      (setq x (cdr x)))
-	    (nreverse result)))
+	  (lambda (x)
+	    (let (elem result)
+	      (while (setq elem (car x))
+		(cond  ((vectorp elem)
+			(setq table-sym
+			      (generate-table-set-table-symbol table-set i))
+			(register-code-conversion-map table-sym (car x))
+			(setq result (cons table-sym result)
+			      i (1+ i)))
+		       ((listp elem)
+			(setq result (cons
+				      (funcall func-get-syms elem)
+				      result)))
+		       ((or (table-set-p elem)
+			    (table-set-table-p elem))
+			(setq result (cons elem result)))
+		       (t
+			(error
+			 "Internal Error:invalid table-set definition:%S"
+			 elem)))
+		(setq x (cdr x)))
+	      (nreverse result))))
 
     (or (get table-set 'table-set-symbol-list)
 	(put table-set 'table-set-symbol-list
 	     (funcall func-get-syms
-		      (get-table-set-definition table-set)
-		      nil)))))
+		      (get-table-set-definition table-set))))))
 
 (defun get-table-set-symbol-list-recursively (table-set)
   "Retuen (nested) symbol list of the tables
@@ -218,31 +217,36 @@ used by the specified table-set.
 This function applies get-table-set-symbol-list
 to all nested table-sets."
   (if (not (table-set-p table-set))
-      (error "%S is not table-set!" table-set))
+  (error "%S is not table-set!" table-set))
 
-  (let (func-get-tables	elem)
+  (let (func-get-tables)
     (setq func-get-tables 
-	  (lambda (x result)
-	    (while (setq elem (car x))
-	      (cond ((table-set-p elem)
-		     (setq result
-			   (nconc result
-				  (get-table-set-symbol-list-recursively
-				   elem))))
-		    ((symbolp elem)
-		     (setq result
-			   (nconc result
-				  (list elem))))
-		    ((listp elem)
-		     (setq result
-			   (nconc result
-				  (funcall
-				   func-get-tables elem nil)))))
-	      (setq x (cdr x)))
-	    result))
+	  (lambda (x)
+	    (let (elem result)
+	      (while (setq elem (car x))
+		(cond ((table-set-p elem)
+		       (setq result
+			     (append result
+				     (get-table-set-symbol-list-recursively
+				      elem))))
+		      ((symbolp elem)
+		       (setq result
+			     (append result
+				     (list elem))))
+		      ((listp elem)
+		       (setq result
+			     (append result
+				     (list
+				      (funcall
+				       func-get-tables elem)))))
+		      (t
+		       (error
+			"Internal Error:invalid table-set element:%S"
+			elem)))
+		(setq x (cdr x)))
+	      result)))
     (funcall func-get-tables
-	     (get-table-set-symbol-list table-set)
-	     nil)))
+	     (get-table-set-symbol-list table-set))))
 
 (defun get-table-set-all-symbols-list (table-set)
   (if (not (table-set-p table-set))
@@ -260,7 +264,7 @@ to all nested table-sets."
 			 (get-table-set-symbol-list current)
 			 rest)))
 	    ((listp current)
-	     (setq rest (nconc current rest)))
+	     (setq rest (append current rest)))
 	    ((symbolp current)
 	     (setq tables (cons current tables)))
 	    (t
@@ -284,7 +288,7 @@ unnecessary tables in order to avoid duplicated table registration."
 		(cons
 		 (cons table (get table 'code-conversion-map))
 		 table-alist))
-	  (mucs-register-object 'table table))
+	  (mucs-register-object 'table table t))
 	(setq tables (cdr tables))))
     table-alist))
 
@@ -303,9 +307,8 @@ unnecessary tables in order to avoid duplicated table registration."
 
 (defun generate-table-set-registration-program (table-set)
   (and (table-set-p table-set)
-       (not (mucs-register-object 'table-set table-set))
        (let* ((syms (get-table-set-all-symbols-list table-set))
-	      (table-sets (car syms))
+	      (table-sets (cons table-set (car syms)))
 	      (table-reg-prog
 	       (generate-tables-registration-program (cdr syms)))
 	      (generate-table-set-reg-prog
@@ -315,21 +318,22 @@ unnecessary tables in order to avoid duplicated table registration."
 				 ts)))
 		   (put (quote ,ts) 'table-set-reference-count
 			(quote ,(get ts 'table-set-reference-count))))))
-	      (table-set-reg-prog
-	       (funcall generate-table-set-reg-prog
-			table-set)))
+	      table-set-reg-prog)
 	 (while table-sets
-	   (setq table-set-reg-prog
-		 (append
-		  table-set-reg-prog
-		  (funcall generate-table-set-reg-prog
-			   (car table-sets)))
-		 table-sets (cdr table-sets)))
+	   (if (not
+		(mucs-register-object 'table-set (car table-sets) t))
+	       (setq table-set-reg-prog
+		     (append
+		      table-set-reg-prog
+		      (funcall generate-table-set-reg-prog
+			       (car table-sets)))))
+	   (setq table-sets (cdr table-sets)))
 	 (if table-reg-prog
 	     `(progn
 		,table-reg-prog
 		,@table-set-reg-prog)
-	   table-set-reg-prog))))
+	   `(progn
+	      ,@table-set-reg-prog)))))
 
 ;;;
 ;;; Table creater
@@ -475,31 +479,7 @@ When CONV is non-nil, convert all elements of alist with CONV."
       (setq curll (cdr curll)))
     (nreverse veclist)))
 
-;;;
-;;; Supplemental functions.
-;;;    These functions may be removed in the future.
-;;;
-
-(defun compile-codepoint-alist-to-vector (table &optional cont func)
-)
-
-(defun merge-codepoint-vector (table11 table2)
-)
-
-(defun generate-meta-table-registration-program (symbol table)
-  (register-code-conversion-map symbol table)
-  (if (mucs-table-registered-p symbol)
-      nil
-    (mucs-add-table-registration symbol)
-    `(let ((tbl ,table)
-	   (i 1)
-	   (j (length tbl))
-	   elem id)
-       (while (< i j)
-	 (setq elem (aref tbl i))
-	 (if (setq id (get 'code-conversion-map-id elem))
-	     (aset tbl i id))
-	 (setq i (1+ i)))
-       (register-code-conversion-map ,symbol tbl))))
-
 (provide 'tbl-mg)
+
+;;; tbl-mg ends here.
+
