@@ -42,37 +42,44 @@
 
 ;;; Required for compile, and also persistent section.
 (eval-and-compile
-  (defvar un-define-basic-translation-charset-order-list
-    '(ascii
-      latin-iso8859-1
-      latin-iso8859-2
-      latin-iso8859-3
-      latin-iso8859-4
-      cyrillic-iso8859-5
+;
+; unicode basic translation charset list.
+;
+
+(defvar unicode-basic-translation-charset-order-list
+  '(ascii
+    latin-iso8859-1
+    latin-iso8859-2
+    latin-iso8859-3
+    latin-iso8859-4
+    cyrillic-iso8859-5
 					;      arabic-iso8859-6
-      greek-iso8859-7
-      hebrew-iso8859-8
-      latin-iso8859-9
-      ipa
-      japanese-jisx0208
-      japanese-jisx0212
-      chinese-gb2312
-      chinese-cns11643-1 
-      chinese-cns11643-2
-      chinese-cns11643-3
-      chinese-cns11643-4
-      chinese-cns11643-5
-      chinese-cns11643-6
-      chinese-cns11643-7
-      korean-ksc5601
-      latin-jisx0201
-      katakana-jisx0201
-      )
-    "*Charset order used by un-define translation")
+    greek-iso8859-7
+    hebrew-iso8859-8
+    latin-iso8859-9
+    ipa
+    japanese-jisx0208
+    japanese-jisx0212
+    chinese-gb2312
+    chinese-cns11643-1 
+    chinese-cns11643-2
+    chinese-cns11643-3
+    chinese-cns11643-4
+    chinese-cns11643-5
+    chinese-cns11643-6
+    chinese-cns11643-7
+    korean-ksc5601
+    latin-jisx0201
+    katakana-jisx0201)
+  "*Charset order used by unicode-basic-translation")
+
+;
+; Default translation rule symbol per charset.
+;
+(defun unicode-get-translation-rule-from-charset (charset)
+  (intern (format "unicode-%s-translation-rule"
+		  (symbol-name charset))))
 
-  (defun un-define-get-translation-from-charset (charset)
-    (intern (format "unicode-%s-translation-rule"
-		    (symbol-name charset))))
 ;;
 ;; Configuration functions for convenience.
 ;;
@@ -101,58 +108,15 @@
 	 (list (cons 'coding-category coding-category)))
      (list (cons 'safe-charsets
 		 (or charsets
-		     un-define-basic-translation-charset-order-list)))))
+		     unicode-basic-translation-charset-order-list)))))
   )
 
 (eval-when-compile
-
   ;; predefined configuration
   ;; Any settings in here affect only
   ;; byte-compiled file.
-  (require 'unicode)
-
-  ;; load-table & declare basic translation rule.
-  ;; translation definition start
-  ;;; basic translation rule, which can be modified dynamically.
-  (tae-declare-translation
-   'unicode-basic-translation-rule
-   `(| ,@(mapcar
-	  (lambda (charset)
-	    (let ((sym
-		   (un-define-get-translation-from-charset
-		    charset)))
-	      (require-unicode-charset-data charset)
-	      (tae-declare-translation
-	       sym
-	       (symbol-value (get charset 'unicode-assoc)))
-	      sym))
-	  un-define-basic-translation-charset-order-list))
-   t) ;; for dynamic modification!
-
-  ;; below translations are static.
-
-  (tae-declare-translation
-   'unicode-basic-stream-translation-rule
-   `(| unicode-basic-translation-rule
-       ,unicode-not-found-to-replace-or-invalid-assoc))
-
-  (tae-declare-translation
-   'unicode-basic-non-stream-translation-rule
-   `(| unicode-basic-translation-rule
-       ,unicode-not-found-to-invalid-assoc))
-
-  (tae-declare-translation
-   'unicode-line-separator-translation-rule
-   `(| ,lf-vs-unicode-line-separator-assoc
-       unicode-basic-stream-translation-rule))
-
-  (tae-declare-translation
-   'unicode-mac-line-separator-translation-rule
-   `(| ,lf-vs-cr-assoc
-       unicode-basic-stream-translation-rule))
-  ;; translation definition ends here
-
-)
+  (require 'un-trbase)
+  (require 'utf))
 
 (mucs-define-package
  un-define
@@ -465,9 +429,9 @@
 		   ,(charset-dimension cs))
 		  (tae-compile 
 		   (quote
-		    ,(un-define-get-translation-from-charset cs)))
+		    ,(unicode-get-translation-rule-from-charset cs)))
 		  (mucs-ccl-flat-code-to-font-encoding 2))))
-	un-define-basic-translation-charset-order-list)))))
+	unicode-basic-translation-charset-order-list)))))
 
 ;;
 ;; coding-system-definition.
@@ -676,18 +640,22 @@
 ; Charset order dynamic modification. (Very simple version!)
 ;
 
+(defun un-define-generate-basic-translation-rule (order-list)
+  `(| ,@(mapcar
+	 (function unicode-get-translation-rule-from-charset)
+	 order-list)))
+
 (defun un-define-change-charset-order (&optional order-list)
-  "Change conversion priority on un-define-basic-translation-rule.
+  "Change conversion priority on unicode-basic-translation-rule.
 ORDER-LIST must be a list that consists of charsets, which specifies
 an order of conversion between internal characters and UCS characters."
   (if (null order-list)
       (setq order-list
-	    un-define-basic-translation-charset-order-list))
+	    unicode-basic-translation-charset-order-list))
   (tae-modify-translation
    'unicode-basic-translation-rule
-   `(| ,@(mapcar
-	  (function un-define-get-translation-from-charset)
-	  order-list))))
+   (un-define-generate-basic-translation-rule
+    order-list)))
 
 (provide 'un-define)
 
