@@ -1,6 +1,6 @@
 ;;; mucs-comp.el --- for byte-compile and other housekeeping jobs.
 
-;; Copyright (C) 1999 Miyashita Hisashi
+;; Copyright (C) 1999-2000 Miyashita Hisashi
 
 ;; Keywords: mule, multilingual, 
 ;;           character set, coding-system, ISO10646, Unicode
@@ -38,20 +38,27 @@
 			       "./lisp/txt-tbl.el"
 			       "./lisp/tae.el"
 			       "./lisp/unicode.el"
+			       "./lisp/utf.el"
 			       "./lisp/un-data.el"
 			       "./lisp/un-tools.el"))
       (compile-file-list-2nd (if command-line-args-left
 				 (prog1
 				     command-line-args-left
 				   (setq command-line-args-left nil))
-			       '("./lisp/un-define.el"))))
+			       '(
+				 "./lisp/un-define.el"
+				 ))))
   (message "Compiling 1st stage-----")
   (mapcar
-   'byte-compile-file
+   (lambda (file)
+     (save-excursion
+       (byte-compile-file file)))
    compile-file-list-1st)
   (message "Compiling 2nd stage!!---")
   (mapcar
-   'byte-compile-file
+   (lambda (file)
+     (save-excursion
+       (byte-compile-file file)))
    compile-file-list-2nd))
 
 
